# computes the autocorrelation of the samples in x, for tau going from 0 to tau.max-1
autocorrelation <- function(x, tau.max) {
	#compute the mean
	mu <- mean(x)
	#center the samples around the mean
	x.norm <- x - mu
	#for convenience, save the number of samples in n
	n <- length(x.norm)
	#initialize an empty vector where to store the autocorrelation as function of tau
	ac <- c()
	#for all values of tau (R vectors are 1-based :-( )
	for (tau in 1:tau.max) {
		#add the autocorrelation for tau to ac as seen in the slides
		ac <- c(ac, 1/(n-tau.max) * sum(x.norm[1:(n-tau.max)] * x.norm[tau:(n-tau.max+tau-1)])
		)
	}
	#return the normalized vector of autocorrelations
	return (ac/ac[1])
}

#plot a set of visualizations for the samples
plot.everything <- function(x, prefix) {

	n <- length(x)

	#plot the samples value
	pdf(paste(prefix, "_samples.pdf", sep=""))
	plot(1:n,x,type='l',xlab="sample",ylab="value")
	dev.off()

	#plot the samples, but sorted
	pdf(paste(prefix, "_ordered_samples.pdf", sep=""))
	plot(1:n,sort(x,decreasing=TRUE),type='l',xlab="sample",ylab="value")
	dev.off()

	#plot histograms for different bin sizes
	for (eps in c(0.001, 0.01,0.1)) {
			breaks = seq(min(x),max(x)+eps,eps)
			pdf(paste(prefix, "_hist_", eps, ".pdf", sep=""))
			hist(x,breaks,xlab="values",main=paste("Histogram with bin width " , eps))
			dev.off()
	}

	#plot the ecdf
	cdf=ecdf(x)
	pdf(paste(prefix, "_ecdf.pdf", sep=""))
	plot(cdf,xlab="value",ylab="ECDF",main="")
	dev.off()

	#plot the autocorrelation
	tau.max <- 100
	ac <- autocorrelation(x, tau.max)
	pdf(paste(prefix, "_wac.pdf", sep=""))
	plot(0:(tau.max-1), ac, xlab="tau", ylab="autocorrelation", type='h', ylim=c(-0.2,1.2))
	points(0:(tau.max-1), ac, col='red', cex=.5, pch=20)
	dev.off()
}

#generate a set of normally distributed samples
generate.gaussian <- function(n, mu, sd, seed=1) {
	set.seed(seed)
	return(rnorm(n, mu, sd))
}
#generate a set of logistically distributed samples
generate.logistic <- function(n, mu, sd, seed=1) {
	set.seed(seed)
	scale <- sd*(3^(1/2))/pi
	return(rlogis(n,mu,scale))
}
#generate samples for a gauss markov process
generate.gauss.markov <- function(n, mu, sd, tau, dt, seed=1) {
	set.seed(seed)
	#compute the std dev for the noise so that the gauss markov process has the required
	#std dev
	sd.noise <- sd*sqrt(1-exp(-2*dt/tau))
	prev.value <- 0
	v <- c(0)
	#compute the samples
	for (i in 2:n) {
		#notice that the new value depends on the previous: correlated process
		new.value <- exp(-dt/tau)*prev.value + rnorm(1, mean=0, sd=sd.noise)
		prev.value <- new.value
		v <- c(v, new.value)
	}
	return (v+mu)
}

#number of samples
n    <- 1000
#mean and standard deviation for all distributions
mean <- 0
sd   <- 1
#time constant and sampling time for the gauss-markov process
tau  <- 0.2
dt   <- 0.01

#generate data and plot graphs for all distributions
x <- generate.gaussian(n, mean, sd)
plot.everything(x, "gaussian")
x <- generate.logistic(n, mean, sd)
plot.everything(x, "logistic")
x <- generate.gauss.markov(n, mean, sd, tau, dt)
plot.everything(x, "gauss_markov")
